{% extends "auth/base.html" %}
{% import "bootstrap/wtf.html" as wtf %}


{% block page_content %}
<div class="page-header">
  <h1>Submit Job</h1>
</div>
<div class="col-md-12">
  <form action="" method="post" class="form" role="form" enctype="multipart/form-data">
    {{ wtf.quick_form(form) }}
  </form>
  <br>
</div>
{% endblock %}
